/******************************************************************************

	FILE				: lcd.c
   
	AUTHOR			: Axiom Manufacturing    					     
						  Spencer Ruggles
						  
	PURPOSE			: Manufacturing Test for EVB51JM128 board, x427
 
 ******************************************************************************
 
	Purpose:
  
	Mfg Functional Test routine for EVB51JM128 board, x427.  

******************************************************************************/

#include "MCF51JM128.h"					// include peripheral declarations 
#include "lcd.h"							// include LCD declarations

#define LCD_DELAYTIME	0X200;		// LCD delay constant 
												// adjust lower for better performance,
												// higher if output is garbled
												
byte LCDBuf;								// LCD data buffer
byte LCDStatus;							// LCD status buffer
 
/*******************************************************************************
LCDInit

	initialize LCD port for 2x8, FEMA, CM082B-SGR1-Z LCD panel.
	LCD port uses SPI1 peripheral
	
	INPUT:		none
	OUTPUT:		none
********************************************************************************/
void LCDInit(){

	SPI1C1 = (SPI1C1_SPE_MASK |SPI1C1_MSTR_MASK | SPI1C1_SSOE_MASK);
												// enable SPI1 as Master w/ SS* output
	SPI1C2 |= SPI1C2_MODFEN_MASK;		// enable SS* output
	SPI1BR |= 0x00;						// set SPI1 prescalar to divide by 1
	
	// intitialize LCD panel
	LCDBuf &= ~(LCD_RS + LCD_EN);		// clr RS and EN to select LCD Command state
	LCDSend();								// send status to LCD
	
	LCD_delaym(50);						// delay to ensure LCD complete command 
	
	lcd_wr_4d(3);							// set to 4bit mode
	lcd_wr_4d(3);							
	lcd_wr_4d(3);							
	lcd_wr_4d(2);							
	
	lcd_cmd(0x2C);							// 2x40 display
	lcd_cmd(0x06);							// display and cursor on
	lcd_cmd(0x0e);							// shift cursor right
	lcd_cmd(0x01);							// clear display and cursor home
	lcd_cmd(0x80);
	
	LCDBuf = 0x00;							// Reset LCD
	LCDSend();								// send reset cmd to LCD
}

/*******************************************************************************
LCD_delayu

	inner delay loop
	uses LCD_DELAYTIME and input count to create a simple delay loop. Use 
	compiler output to determine # of clock cycles used.  Incr or Decr count to  tune
	fine delay
	
	INPUT:		ucount
	OUTPUT:		none
********************************************************************************/
void LCD_delayu(byte ucount) {
	while(ucount) {
		--ucount;							// decrement inner loop count
	}
}

/*******************************************************************************
LCD_delaym

	outer delay loop.  calls LCD_delayu to create LCD delay
	
	
	INPUT:		mcount
	OUTPUT:		none
********************************************************************************/
void LCD_delaym(byte mcount) {
	while(mcount) {
		--mcount;							// decrement outer loop count
	}
	LCD_delayu;								// call inner loop
}

/*******************************************************************************
LCDSend

	simple SPI serial driver to send a byte of data.  sends data byte in global
	LCDBuf buffer and stores received value in global LCDStatus buffer
	
	
	INPUT:		none
	OUTPUT:		none
********************************************************************************/
void LCDSend() {
	
	LCD_delaym(1);							// delay
	LCDStatus = SPI1S;					// clr LCD status by reading SR
	SPI1DL = LCDBuf;						// send data
	
//	do {
//		LCDStatus = SPI1S;				// get LCD status
//	} while((LCDStatus &0x80) == 0x00);	// wait until status flag sets
//	while(!SPI1S_SPRF);					// wait for status flag to set
	
	LCDStatus = SPI1DL;					// read data back
}

/*******************************************************************************
lcd_wr_4

	writes 4 bytes of data to LCD port.  Pattern used to set and hold data while 
	control bits toggle
	
	
	INPUT:		LCDdata
	OUTPUT:		none
********************************************************************************/
void lcd_wr_4(byte LCDdata) {

	LCDBuf &= 0xF0;						// merge lower 4b of LCDdata with upper
	LCDBuf |= LCDdata;					// 4b of LCDBuf (control bits)
	
	LCDBuf &= !LCD_EN;					// set LCD enable low
	LCDSend();								// send data
	LCDBuf &= LCD_EN;						// set LCD enable low
	LCDSend();								// send data
	LCDBuf &= !LCD_EN;					// set LCD enable low
	LCDSend();								// send data
}

/*******************************************************************************
lcd_wr_4d

	same as lcd_wr_4 above with longer delay
	
	
	INPUT:		LCDdata
	OUTPUT:		none
********************************************************************************/
void lcd_wr_4d(byte LCDdata) {
	
	lcd_wr_4(LCDdata);					// send byte of data
	LCD_delaym(50);						// delay
}

/*******************************************************************************
LCD_Write

	write 8bits of data, upper 4bits 1st then lower 4bits
	
	
	INPUT:		LCDval	LCD data value
	OUTPUT:		none
********************************************************************************/
void LCD_Write(byte lcdval) {

	lcd_wr_4(lcdval >> 4);				// send upper 4 bits
	lcd_wr_4(lcdval & 0x0F);			// send lower 4 bits
}

/*******************************************************************************
lcd_cmd

	write command byte to LCD 
	
	
	INPUT:		LCDval	LCD data value
	OUTPUT:		none
********************************************************************************/
void lcd_cmd(byte cmdval) {
	
	LCDBuf &= !LCD_RS;					// clear to RS to select command mode
	LCD_Write(cmdval);					// send command value
	LCD_delaym(10);						// delay
}

/*******************************************************************************
LCDputch

	send a char to LCD for display 
	
	
	INPUT:		datval	char to display
	OUTPUT:		none
********************************************************************************/
void LCDputch(byte datval) {
	
	LCDBuf |= LCD_RS;						// set LCD to data mode
	LCD_Write(datval);					// write char to LCD
}

/*******************************************************************************
LCDputs

	send a string of char to LCD for display 
	
	
	INPUT:		sptr		pointer to string of char to display
	OUTPUT:		none
********************************************************************************/
void LCDputs(byte *sptr) {
	while(*sptr) {
		LCDputch(*sptr);					// send char data
		++ sptr;								// get next char
	}
}



