/******************************************************************************

	FILE				: sci.c
   
	AUTHOR			: Axiom Manufacturing    					     
						  Spencer Ruggles
						  
	PURPOSE			: Manufacturing Test for EVB51JM128 board, x427
 
 ******************************************************************************
 
	Purpose:
  
	Mfg Functional Test routine for EVB51JM128 board, x427.  

******************************************************************************/

#include "MCF51JM128.h"					// include peripheral declarations 
#include "sci.h"							// sci header file
#include "textstr.h"						// text string header file

/*******************************************************************************
display

	outputs formatted display to COM1 for test purposes 
********************************************************************************/
void display_menu(byte port){
	puts(port, axstr);					// send string to COM1, array name points to start of array
	puts(port, bdstr);
	putchar(port,'\n');					// send CR/LF to COM1
	putchar(port,'\n');					// send CR/LF to COM1
	
	puts(port, menu1);					// menu item #1
	puts(port, menu2);					// menu item #2
	puts(port, menu3);					// menu item #3
	puts(port, menu4);					// menu item #4
	puts(port, menu5);					// menu item #5
	puts(port, menu6);					// menu item #6
	puts(port, menu7);					// menu item #7
	puts(port, menu8);					// menu item #8
	puts(port, menu9);					// menu item #9

	putchar(port,'\n');					// send CR/LF to COM1
	putchar(port,'\n');					// send CR/LF to COM1
	
	puts(port,select);					// Enter selection string
}
	
/*******************************************************************************
PressAnyKey

	sends string "Press Any Key to Continue" to selected COM port 
********************************************************************************/
void PressAnyKey(byte port){
	putchar(port,'\n');					// send CR/LF to COM1
	putchar(port,'\n');					// send CR/LF to COM1
	puts(port,press);							// send string to COM port
}

/*******************************************************************************
ensci(port)

	configures COM port selected by (port) for 19200,8,N,1 operation
	SCI baud rate register (SCIxBD) is written with value determined by 
	     SCIxBD = busclk/(16*baud) => SCIxBD = 0x0D -> 19.2 kHz
	     
	MCG trim value must be written to effectively use COM port 
	
	INPUT:	port		0=SCI1, 1=SCI2
	OUTPUT:	none
********************************************************************************/
void ensci(byte port){
	if(!port) {		   						// COM1 selected
		SCI1BD = baud19;						// 19.2 kHz
		SCI1C1 &= ~SCI1C1_M_MASK & ~SCI1C1_PE_MASK;	// 8,N,1
		SCI1C2 |= SCI1C2_TE_MASK | SCI1C2_RE_MASK;	// SCI Tx/Rx enabled
	} else {										// COM2 selected
		SCI2BD = baud19;						// 19,200 Hz
		SCI2C1 &= ~SCI2C1_M_MASK & ~SCI2C1_PE_MASK;	// 8,N,1
		SCI2C2 |= SCI2C2_TE_MASK | SCI2C2_RE_MASK;	// SCI Tx/Rx enabled
	}
}

/*******************************************************************************
ReadyToSend

	wait for selected COM port Tx Data Reg Empty flag to set indicating tx is 
	ready for a byte of data
	
	INPUT:	port		0=SCI1, 1=SCI2
	OUTPUT:	returns 	0 when ready - no TIMEOUE feature implemented
********************************************************************************/
byte ReadyToSend(byte port) {
	byte status;							// temp var
	
	do {
		if(port){
			status = SCI2S1;				// get COM2 TDRE flag status
		} else {								
			status = SCI1S1;				// get COM1 TDRE flag status
		} 
	}while((status & 0x80) == 0);		// loop until TDRE flag is set
	return 0;								// return 0 
}

/*******************************************************************************
putchar
	check for ReadyToSend then output byte of data on selected COM port
	
	INPUT:	port		0=SCI1, 1=SCI2
				sbyte 	output data to write to COM port
	OUTPUT:	returns 	0 when ready - no TIMEOUE feature implemented
********************************************************************************/
byte putchar(byte port, byte sbyte) {
	if(ReadyToSend(port)) return 1;	// wait for TDRE flag to set
	
	if(port) {								// if COM2 selected
		SCI2D = sbyte;						// send data byte on COM2
		while(!SCI2S1_TC);				// wait for Tx Complete flag to set
	} else {
		SCI1D = sbyte;						// send data byte on COM1
		while(!SCI1S1_TC);				// wait for Tx Complete flag to set
	}
	return 0;								// return successful
}

/*******************************************************************************
putch

	send a single byte of data out selected COM port.  if input byte is '\n' char
	then send carriage return and line feed (CR\LF)
	
	INPUT:	port		0=SCI1, 1=SCI2
				sbyte 	output data to write to COM port
	OUTPUT: none
********************************************************************************/
void putch(byte port, byte ch){
	if(ch == '\n'){	 				// if \n send CR/LF
		putchar(port, CR);
		putchar(port, LF);
	} else {
		putchar(port, ch);					// send data byte
	}
}

/*******************************************************************************
puts

	send a string of data out selected COM port with CR/LF.  string should be null 	
	terminated
	
	INPUT:		port		0=SCI1, 1=SCI2
					sptr		pointer to data string
	OUTPUT: none
********************************************************************************/
void puts(byte port, byte *sptr) {
	while(*sptr) {						// if data still in string
		putch(port, *sptr);			// send data to selected COM port w/ CR/LF
		++sptr;
	}
}

/*******************************************************************************
fputs

	send a string of data out selected COM port with CR/LF.  string should be null 	
	terminated
	
	INPUT:		port		0=SCI1, 1=SCI2
					sptr		pointer to data string
	OUTPUT: none
********************************************************************************/
void fputs(byte port, byte *sptr) {
	byte ch;								
	while(*sptr) {						// if data still in string
		ch = *sptr;						// get data byte
//		if(ch="\n") break;
		putchar(0,ch);					// send data
//		putch(port, *sptr);			// send data to selected COM port w/ CR/LF
		++sptr;
	}
}

/*******************************************************************************
SerialReady

	returns 0 in new serial data avaiable on selected COM port
	
	INPUT:		port		0 = SCI1, 1 = SCI2
	OUTPUT: 		status	0 = data available,  1 = no data avaiable
********************************************************************************/
byte SerialReady(byte port){
	byte status;						// temp variable
	
	if(port) status = SCI2S1;		// get status on COM2
	else status = SCI1S1;			// get status on COM1

	if(status & 0x20) return 0;	// return 0 if new data available
	else return 1;						// return 1 if no data available
}

/*******************************************************************************
ReadSerialPort

	reads a single byte from selected serial port
	
	INPUT:		port		0 = SCI1, 1 = SCI2
	OUTPUT: 		returns the data read from selected COM port 
********************************************************************************/
byte ReadSerialPort(byte port){
	byte sbyte;							// temp var
	
	if(port) sbyte = SCI2D;			// get COM2 data
	else sbyte = SCI1D;				// get COM1 data
	
	return sbyte;						// return data byte read
}

/*******************************************************************************
GetSByte

	wait for Rx Data Ready Flag (RDRF) to set calls ReadSerialPort to get data
	from selected serial port
	
	INPUT:		port		0 = SCI1, 1 = SCI2
	OUTPUT: 		returns the data read from selected COM port 
********************************************************************************/
byte GetSByte(byte port){
	
	while(SerialReady(port)); 		// wait for new input data
	return(ReadSerialPort(port));	// read input data on selected COM port
}

/*******************************************************************************
hex2char


	input: 4-bit hex value in low nibble of input
	outpt: ASCII representation of input
********************************************************************************/
byte hex2char(byte hval) {
	if(hval < 0x0A) {
		hval += 0x30;						// convert high nibble numeric to ASCII text 
	} else {
		hval += 0x37;						// convert high nibble alpha to ASCII text
	}
	return hval;							// return ASCII char
}

/*******************************************************************************
char2hex

	converts an input  value to its hex value.  ONLY CONVERTS ASCII NUMBERS. 
	
	input: 8-bit ASCII char 
	outpt: hex byte
********************************************************************************/
byte char2hex(byte cval){
	if(cval > 0x30 && cval < 0x3A){
		cval -= 0x30;						// convert char to hex
	}
	return cval;							// return hex val
}

/*******************************************************************************
outhex

	formats 8-bit hex value into ASCII for display on selected COM port
	
	input: 8-bit hex value 
	outpt: ASCII representation of input to selected COM port
********************************************************************************/
void outhex(byte port, byte hval){
	// convert input HEX value to output ASCII text

	unsigned char unib,lnib;			// temp variable for upper and lower nibbles
	
	unib = (hval >> 4);					// high nibble
	lnib = hval & 0x0F;					// low nibble

	putch(port, hex2char(unib));		// send char out selected COM port
	putch(port, hex2char(lnib));		// send char out selected COM port
}  

/*******************************************************************************
cls
	
	clears COM dispay by sending 80 lines of CR/LF
	
	INPUT:		port		0=SCI1, 1=SCI2
	outpt: 		none
********************************************************************************/
void cls(byte port){
	byte n;									// temp variable
	
	for(n=0;n<80;++n){
		putchar(port,'\n');				// send CR/LF 
	}
}
/*******************************************************************************
DisplayPOT
	
	formats POT analog result for display
	
	INPUT:		port		0=SCI1, 1=SCI2
	outpt: 		none
********************************************************************************/
void DisplayATD(byte port){
	fputs(0, ATDstr);						// send formatted string
	outhex(0,ADCRL);						// send results
}
